@@ -1127,7 +1127,7 @@ convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 	}
 	for (i = 0; i < decimal - sign; i++) {
 		/* wa->abacus.displayBase == wa->abacus.base and all that ... */
-		digit = char2Int(string[decimal - 1 - i]);
+		digit = char2Int(string[decimal - 1 - i], base);
 		if (bottomNumber < base - 1 &&
 				base % topFactor != 0 && diff >= 0 &&
 				digit >= base - 1 - diff) { /* Cn16 */
@@ -1164,7 +1164,7 @@ convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 		int offset = 0;
 		int j;
 
-		digit = char2Int(string[decimal + i + 1]);
+		digit = char2Int(string[decimal + i + 1], base);
 		if  (wa->abacus.decks[BOTTOM].piece != 0)
 			offset++;
 		if  (wa->abacus.decks[BOTTOM].piecePercent != 0 &&
@@ -1185,7 +1185,6 @@ convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 						setAbacusMove(w, ACTION_MOVE, aux, 1, j, topNumber);
 						digit = digit - topNumber;
 					}
-						
 				} else {
 					factor = digit / topFactor;
 					if (factor > 0) {
