@@ -27,9 +27,9 @@ const char *bookTextFallback[1][LINES] =
 {
 	{
 		"  Abacus Demo by Luis Fernandes and David Bagley",
-		"Type: `r' to Read this book",
-		"      `n' to Next book",
-		"      `q' to Quit the demo",
+		READ_BOOK,
+		NEXT_BOOK,
+		QUIT,
 	},
 };
 
@@ -38,44 +38,44 @@ const char *chapterTextFallback[1][5][LINES] =
 	{
 		{
 			"     1. Increment Chapter",
-			"Type: `r' to Read this chapter",
-			"      `n' to Next chapter",
-			"      `q' to Quit the demo",
+			READ_CHAPTER,
+			NEXT_CHAPTER,
+			QUIT,
 		}, {
 			"     2. Decrement Chapter",
-			"Type: `r' to Read this chapter",
-			"      `n' to Next chapter",
-			"      `q' to Quit the demo",
+			READ_CHAPTER,
+			NEXT_CHAPTER,
+			QUIT,
 		}, {
 			"     3. Addition Chapter",
-			"Type: `r' to Read this chapter",
-			"      `n' to Next chapter",
-			"      `q' to Quit the demo",
+			READ_CHAPTER,
+			NEXT_CHAPTER,
+			QUIT,
 		}, {
 			"     4. Subtraction Chapter",
-			"Type: `r' to Read this chapter",
-			"      `n' to Next chapter",
-			"      `q' to Quit the demo",
+			READ_CHAPTER,
+			NEXT_CHAPTER,
+			QUIT,
 		}, {
 			"     5. Multiplication Chapter",
-			"Type: `r' to Read this chapter",
-			"      `n' to Next chapter",
-			"      `q' to Quit the demo",
+			READ_CHAPTER,
+			NEXT_CHAPTER,
+			QUIT,
 		/*}, {
 			"     6. Division Chapter",
-			"Type: `r' to Read this chapter",
-			"      `n' to Next chapter",
-			"      `q' to Quit the demo",
+			READ_CHAPTER,
+			NEXT_CHAPTER,
+			QUIT,
 		}, {
 			"     7. Square Root Chapter",
-			"Type: `r' to Read this chapter",
-			"      `n' to Next chapter",
-			"      `q' to Quit the demo",
+			READ_CHAPTER,
+			NEXT_CHAPTER,
+			QUIT,
 		}, {
 			"     8. Cube Root Chapter",
-			"Type: `r' to Read this chapter",
-			"      `n' to Next chapter",
-			"      `q' to Quit the demo",*/
+			READ_CHAPTER,
+			NEXT_CHAPTER,
+			QUIT,*/
 		},
 	},
 };
