@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusM.c
  *
- * Copyright 1994 - 2018  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1994 - 2021  David A. Bagley, bagleyd AT verizon.net
  *
  * All rights reserved.
  *
@@ -32,15 +32,15 @@
 #define MAX_ROMAN_DIGIT 1000000
 #define ROMANFRACTIONBASE 12
 #define MAX_OLDROMAN 9
-
+#define UPPERCASE_BASE 36
 static char roman[] =
 {
 	'I', 'V', 'X', 'L', 'C', 'D', 'M',
 	     'v', 'x', 'l', 'c', 'd', 'm'
 };
 static char oldRoman[][MAX_OLDROMAN] = {
-	"I", "V",     "X",    "L",       "(",    "I)", "(I)",
-	   "I))", "((I))", "I)))", "(((I)))", "I))))", "|x|"
+	"I", "V", "X", "L", "(", "I)", "(I)",
+	"I))", "((I))", "I)))", "(((I)))", "I))))", "|x|"
 };
 /*
 Here we have lower case to represent the letters with bars on top.
@@ -123,27 +123,28 @@ static char subeighthGlyphs[][MAX_SUBEIGHTH_GLYPHS] = {
 	"", "t", "Q", "Qt"};
 
 int
-char2Int(char character)
+char2Int(char character, int base)
 {
 	int charValue = character;
+	int caseBase = (base > UPPERCASE_BASE) ? 26 : 0;
 
 	if (charValue >= '0' && charValue <= '9') {
 		charValue += 0 - '0';
 	/* ASCII or EBCDIC */
-	} else if (charValue >= 'a' && charValue <= 'i') {
-		charValue += 10 - 'a';
-	} else if (charValue >= 'j' && charValue <= 'r') {
-		charValue += 19 - 'j';
-	} else if (charValue >= 's' && charValue <= 'z') {
-		charValue += 28 - 's';
 	} else if (charValue >= 'A' && charValue <= 'I') {
 		charValue += 10 - 'A';
 	} else if (charValue >= 'J' && charValue <= 'R') {
 		charValue += 19 - 'J';
 	} else if (charValue >= 'S' && charValue <= 'Z') {
 		charValue += 28 - 'S';
+	} else if (charValue >= 'a' && charValue <= 'i') {
+		charValue += caseBase + 10 - 'a';
+	} else if (charValue >= 'j' && charValue <= 'r') {
+		charValue += caseBase + 19 - 'j';
+	} else if (charValue >= 's' && charValue <= 'z') {
+		charValue += caseBase + 28 - 's';
 	} else
-		charValue = 36;
+		charValue = caseBase + UPPERCASE_BASE;
 	return charValue;
 }
 
@@ -159,6 +160,12 @@ int2Char(int digit)
 			charValue = (char) (charValue + 'J' - 'I' - 1);
 		if (charValue > 'R')
 			charValue = (char) (charValue + 'S' - 'R' - 1);
+		if (charValue > 'Z')
+			charValue = (char) (charValue + 'a' - 'Z' - 1);
+		if (charValue > 'i')
+			charValue = (char) (charValue + 'j' - 'i' - 1);
+		if (charValue > 'r')
+			charValue = (char) (charValue + 's' - 'r' - 1);
 	}
 	return charValue;
 }
@@ -215,6 +222,10 @@ flt2String(char *buf, LONG number, int abacusBase, int base, int places,
 		digit = (int) (fraction * divisor);
 		fraction -= mult * digit;
 		buf[position] = int2Char(digit);
+		if (position == decimalPosition - 1
+				&& fraction * divisor >= 5) {
+			buf[position] = int2Char(digit + 1);
+		} else
 		mult /= base;
 		divisor *= base;
 		if (mult == 0 && position != places - 1) {
@@ -223,6 +234,32 @@ flt2String(char *buf, LONG number, int abacusBase, int base, int places,
 		}
 	}
 	buf[position] = '\0';
+	digit = (int) (fraction * divisor);
+	/* Rounding,  what to do if base odd? */
+	if (digit >= base / 2 && position == decimalPosition) {
+		int place = position - 1;
+
+		/* Make sure not all (base-1) or will break */
+		for (digit = position - 1; digit > 0; digit--) {
+			int digitValue = char2Int(buf[digit], base);
+			if (digitValue >= base - 1) {
+				if (digit == 0)
+					place = 0;
+			} else
+				break;
+		}
+		/* Add one */
+		for (digit = place; digit > 0; digit--) {
+			int digitValue = char2Int(buf[digit], base);
+
+			if (digitValue >= base - 1)
+				buf[digit] = '0';
+			else {
+				buf[digit] = int2Char(digitValue + 1);
+				break;
+			}
+		}
+	}
 	for (digit = position - 1; digit > 0; digit--) {
 		if (buf[digit] == '0')
 			buf[digit] = '\0';
@@ -246,7 +283,7 @@ string2Int(char *buf, int base, char decimalPoint)
 		if (buf[position] == decimalPoint) {
 			break;
 		}
-		digit = char2Int(buf[position]);
+		digit = char2Int(buf[position], base);
 		value = value * base + digit;
 		if (value >= ((base >= 8) ? base / 2 - 1 : base) *
 				base * base * base * base * base * base)
@@ -431,15 +468,15 @@ addStrings(char *buf, char *aBuf, char *bBuf, int base, char decimalPoint)
 	}
 	if (aDecimal + aSign >= bDecimal + bSign) {
 		for (i = aEnd - 1; i > aDecimal; i--) {
-			digit = char2Int(aBuf[i]) +
-				char2Int(bBuf[i - aEnd + bEnd]) + carry;
+			digit = char2Int(aBuf[i], base) +
+				char2Int(bBuf[i - aEnd + bEnd], base) + carry;
 			aBuf[i] = int2Char(digit % base);
 			carry = digit / base;
 		}
 		for (i = aDecimal - 1; i >= aSign; i--) {
-			digit = char2Int(aBuf[i]) +
+			digit = char2Int(aBuf[i], base) +
 				((i - aEnd + bEnd >= bSign) ?
-				char2Int(bBuf[i - aEnd + bEnd]) : 0) + carry;
+				char2Int(bBuf[i - aEnd + bEnd], base) : 0) + carry;
 			aBuf[i] = int2Char(digit % base);
 			carry = digit / base;
 		}
@@ -454,15 +491,15 @@ addStrings(char *buf, char *aBuf, char *bBuf, int base, char decimalPoint)
 			(void) strncpy(buf, aBuf, (unsigned int) (aEnd + 1));
 	} else {
 		for (i = bEnd - 1; i > bDecimal; i--) {
-			digit = char2Int(bBuf[i]) +
-				char2Int(aBuf[i - bEnd + aEnd]) + carry;
+			digit = char2Int(bBuf[i], base) +
+				char2Int(aBuf[i - bEnd + aEnd], base) + carry;
 			bBuf[i] = int2Char(digit % base);
 			carry = digit / base;
 		}
 		for (i = bDecimal - 1; i >= bSign; i--) {
-			digit = char2Int(bBuf[i]) +
+			digit = char2Int(bBuf[i], base) +
 				((i - bEnd + aEnd >= aSign) ?
-				char2Int(aBuf[i - bEnd + aEnd]) : 0) + carry;
+				char2Int(aBuf[i - bEnd + aEnd], base) : 0) + carry;
 			bBuf[i] = int2Char(digit % base);
 			carry = digit / base;
 		}
@@ -485,6 +522,31 @@ addStrings(char *buf, char *aBuf, char *bBuf, int base, char decimalPoint)
 #endif
 }
 
+static int
+logInt(double x, int base, int power)
+{
+        double quotient = x;
+        int i = 0;
+
+	for (i = 0; i < power - 1; i++)
+		quotient *= x;
+	i = 0;
+        if (quotient >= base) {
+                while (quotient >= base) {
+                        quotient /= base;
+                        i++;
+                }
+                if (quotient + 0.0000000001 >= base)
+                        i++;
+        } else if (quotient > 0.0) {
+                while (quotient < 1.0) {
+                        quotient *= base;
+                        i--;
+                }
+        }
+        return i + 2;
+}
+
 void
 convertString(char * buf, char * inbuf, int base, int displayBase,
 		int decimalPosition,
@@ -496,9 +558,11 @@ convertString(char * buf, char * inbuf, int base, int displayBase,
 	int i, last, place = -1, decimalPlace, decimalPointPlace, mult;
 	Boolean negative, gotDecimal = False;
 	LONG intPart = 0, floatPart = 0, tmpPower;
+	/* decimal point should scale with log(base)/log(displayBase) */
+	int decimalPointFactor = logInt(base, displayBase, decimalPosition);
 
 #ifdef DEBUG
-	(void) printf("convertString: inbuf %s\n", inbuf);
+	(void) printf("convertString: inbuf %s \n", inbuf);
 #endif
 	last = (int) strlen(inbuf);
 	negative = (inbuf[0] == '-');
@@ -519,7 +583,7 @@ convertString(char * buf, char * inbuf, int base, int displayBase,
 			floatPart = 0;
 			place--;
 		} else if (gotDecimal) {
-			mult = char2Int(inbuf[i]);
+			mult = char2Int(inbuf[i], base);
 			tmpPower = multPower(mult, base, place);
 			if (tmpPower < 0) {
 				floatPart = 0;
@@ -528,7 +592,7 @@ convertString(char * buf, char * inbuf, int base, int displayBase,
 			floatPart += tmpPower;
 			place--;
 		} else {
-			mult = char2Int(inbuf[i]);
+			mult = char2Int(inbuf[i], base);
 			if (place >= shiftAnomaly && anomaly != 0) {
 			  if (place >= shiftAnomaly + shiftAnomalySq &&
 						anomalySq != 0) {
@@ -586,7 +650,7 @@ convertString(char * buf, char * inbuf, int base, int displayBase,
 	}
 	(void) int2String(buf, intPart, displayBase, negative);
 	(void) flt2String(fltbuf, floatPart, base, displayBase,
-		decimalPointPlace, decimalPosition);
+		decimalPointPlace, decimalPointFactor);
 	(void) strncat(buf, ".", 2);
 	(void) strncat(buf, fltbuf, STRING_SIZE);
 #ifdef DEBUG
@@ -612,14 +676,20 @@ romanFraction(char *buf, int base, int number, int subnumber,
 		int subbase, Boolean latin)
 {
 	Boolean gotFraction = False;
-	int halfBase = subbase / 2;
+	int halfBase = subbase >> 1;
 	int fraction = number, subfraction = subnumber;
 
+	if (subfraction >= subbase && subbase >= 2) {
+		fraction = number + subfraction / subbase;
+		subfraction %= subbase;
+		if (fraction >= base)
+			fraction %= base;
+	}
 	if (base < ROMANFRACTIONBASE) {
 		fraction *= ROMANFRACTIONBASE / base;
 		subfraction *= ROMANFRACTIONBASE / base;
 		fraction += (subfraction / subbase);
-		subfraction = subfraction % subbase;
+		subfraction %= subbase;
 	}
 	if (fraction == 1 && subfraction >= halfBase) {
 		subfraction -= halfBase;
@@ -637,7 +707,7 @@ romanFraction(char *buf, int base, int number, int subnumber,
 			(void) strncat(buf, twelfthGlyphs[fraction], MAX_TWELFTH_GLYPHS);
 		}
 	}
-	if (subfraction >= halfBase) {
+	if (subfraction >= halfBase && halfBase > 0) {
 		subfraction -= halfBase;
 		if (latin) {
 			if (gotFraction)
@@ -652,12 +722,16 @@ romanFraction(char *buf, int base, int number, int subnumber,
 		if (latin) {
 			if (gotFraction)
 				(void) strncat(buf, " ", 2);
-			if (subbase == 8)
+			if (subbase == 4)
+				(void) strncat(buf, subeighthStrings[2 * subfraction], MAX_SUBEIGHTH);
+			else if (subbase == 8)
 				(void) strncat(buf, subeighthStrings[subfraction], MAX_SUBEIGHTH);
 			else
 				(void) strncat(buf, subtwelfthStrings[subfraction], MAX_SUBTWELFTH);
 		} else {
-			if (subbase == 8)
+			if (subbase == 4)
+				(void) strncat(buf, subeighthGlyphs[2 * subfraction], MAX_SUBEIGHTH_GLYPHS);
+			else if (subbase == 8)
 				(void) strncat(buf, subeighthGlyphs[subfraction], MAX_SUBEIGHTH_GLYPHS);
 			else
 				(void) strncat(buf, subtwelfthGlyphs[subfraction], MAX_SUBTWELFTH_GLYPHS);
@@ -705,7 +779,7 @@ string2Roman(char *buf, char *inbuf, int base, Boolean decimalFraction,
 		}
 	}
 	i = 0;
-	digit = char2Int(inbuf[i]);
+	digit = char2Int(inbuf[i], base);
 	if (last > loga || (last == loga &&
 			digit >= ((base >= 8) ? base / 2 - 1 : base))
 			|| decimalFraction) {
@@ -719,7 +793,7 @@ string2Roman(char *buf, char *inbuf, int base, Boolean decimalFraction,
 	for (i = 0; i < last; i++) {
 		int romanChar = 2 * (last - i) - 1;
 
-		digit = char2Int(inbuf[i]);
+		digit = char2Int(inbuf[i], base);
 		/* IX */
 		if (!ancientRoman && digit >= base + 1 - base / 4 &&
 				base >= 8) {
