@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusP.h
  *
- * Copyright 1994 - 2021  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1994 - 2022  David A. Bagley, bagleyd AT verizon.net
  *
  * Abacus demo and neat pointers from
  * Copyright 1991 - 1998  Luis Fernandes, elf AT ee.ryerson.ca
@@ -67,23 +67,20 @@ extern void complementAbacus(AbacusWidget w);
 extern void incrementAbacus(AbacusWidget w);
 extern void decrementAbacus(AbacusWidget w);
 extern void changeFormatAbacus(AbacusWidget w);
-extern void changeMuseumAbacus(AbacusWidget w);
-extern void changeRomanNumeralsAbacus(AbacusWidget w);
 extern void toggleLatinAbacus(AbacusWidget w);
 extern void toggleGroupingAbacus(AbacusWidget w);
 extern void togglePlaceOnRailAbacus(AbacusWidget w);
 extern void toggleVerticalAbacus(AbacusWidget w);
 extern void toggleNegativeSignAbacus(AbacusWidget w);
-extern void toggleQuartersAbacus(AbacusWidget w);
-extern void toggleQuarterPercentsAbacus(AbacusWidget w);
-extern void toggleTwelfthsAbacus(AbacusWidget w);
-extern void toggleSubdecksAbacus(AbacusWidget w);
 extern void changeRomanMarkersAbacus(AbacusWidget w);
-extern void toggleEighthsAbacus(AbacusWidget w);
+extern void changePieceAbacus(AbacusWidget w);
+extern void changePiecePercentAbacus(AbacusWidget w);
+extern void changeSubdecksAbacus(AbacusWidget w);
+extern void changeRomanNumeralsAbacus(AbacusWidget w);
 extern void toggleSubdecksSeparatedAbacus(AbacusWidget w);
 extern void toggleAltSubbeadPlacementAbacus(AbacusWidget w);
-extern void toggleAnomalyAbacus(AbacusWidget w);
-extern void toggleWatchAbacus(AbacusWidget w);
+extern void changeMuseumAbacus(AbacusWidget w);
+extern void changeAnomalyAbacus(AbacusWidget w);
 extern void speedUpAbacus(AbacusWidget w);
 extern void slowDownAbacus(AbacusWidget w);
 extern void toggleSoundAbacus(AbacusWidget w);
@@ -149,6 +146,7 @@ extern AbacusClassRec abacusClassRec;
 
 #define MAX_DECKS 2
 #define PLACE_SETTING 2
+#define COMPLEMENT 3
 #define UP 1
 #define DOWN 0
 #define TOP 1
@@ -165,7 +163,7 @@ extern AbacusClassRec abacusClassRec;
 #define READ_BOOK	"Hit: `Enter' or `e' to read this book"
 #define NEXT_BOOK	"     `Tab' or `n' to skip to the next book"
 #define READ_CHAPTER	"Hit: `Enter' or `e' to read this chapter"
-#define NEXT_CHAPTER	"     `Tab' or `n' to goto the next chapter"
+#define NEXT_CHAPTER	"     `Tab' or `n' to go to the next chapter"
 #define NEXT_LESSON	"Hit: `Tab' or `n' for the next lesson"
 #define READ_LESSON	"     `Enter' or `e' to repeat lesson"
 #define JUMP_LESSON	"     `j' to jump to next chapter"
@@ -261,7 +259,7 @@ typedef struct _AbacusPart {
 	Boolean     mono, reverse, focus, vertical;
 	Boolean     script, demo, teach, lee;
 	Boolean     latin, slot, diamond;
-	Boolean     subdecksSeparated, altSubbeadPlacement; 
+	Boolean     subdecksSeparated, altSubbeadPlacement;
 	Boolean     versionOnly, sound;
 	char       *digits, *mathBuffer;
 	FILE       *fp;
@@ -276,7 +274,7 @@ typedef struct _AbacusPart {
 	int         reg, regCount, qDigit, qPosition, auxCount, primarySteps;
 	double      divisor;
 	char        op;
-	int        *intGroup;
+	int        *digitGroup;
 	struct _AbacusPart *leftAux, *rightAux;
 #ifdef WINVER
 	char        museum[COUNTRY_SIZE];
@@ -384,7 +382,7 @@ extern void setAbacusMove(AbacusWidget w, int reason, int aux,
 extern void setAbacusHighlightRail(AbacusWidget w, int reason, int aux, int rail);
 extern void setAbacusHighlightRails(AbacusWidget w, int aux);
 extern void setAbacusText(AbacusWidget w, int reason, char * text, int line);
-extern int char2Int(char character);
+extern int char2Int(char character, int base);
 extern char int2Char(int digit);
 extern char *strAddChar(char *buf, char c);
 extern char *strAddInt(char *buf, int value);
@@ -440,6 +438,7 @@ extern void convertStringToAbacus(AbacusWidget w, const char *string, int aux);
 extern double convertToDecimal(int base, char *inputString);
 extern void convertFromInteger(char *outputString, int base, int x);
 extern void convertFromDouble(char *outputString, int base, double x);
+extern void initializeText(AbacusWidget w);
 #ifdef WINVER
 extern void drawDemoText(const char* line, int i);
 extern void drawTeachText(const char* line, int i);
