@@ -1,7 +1,7 @@
 /*
  * @(#)xwin.h
  *
- * Copyright 2005 - 2013  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 2005 - 2017  David A. Bagley, bagleyd AT verizon.net
  *
  * All rights reserved.
  *
@@ -35,7 +35,7 @@
 #include <string.h>
 #include <math.h>
 
-#if ((WINVER > 0x030a) && !defined(GCL_HBRBACKGROUND))
+#if ((WINVER > 0x030a) && !defined(GCL_HBRBACKGROUND) && !defined(GCLP_HBRBACKGROUND))
 #undef WINVER
 #define WINVER 0x030a
 #endif
@@ -45,7 +45,11 @@
 #define MoveTO(h,x,y,lp) MoveTo(h,x,y)
 #else
 #define Position long
+#ifdef GCLP_HBRBACKGROUND
+#define SETBACK(h,b) (void)SetClassLongA(h,GCLP_HBRBACKGROUND,(LONG)(intptr_t)b)
+#else
 #define SETBACK(h,b) (void)SetClassLong(h,GCL_HBRBACKGROUND,(LONG)b)
+#endif
 #define MoveTO(h,x,y,lp) MoveToEx(h,x,y,lp)
 #endif
 #ifndef Point
@@ -70,6 +74,8 @@
 #define Pixmap HBITMAP
 #define FULL_CIRCLE NUM_DEGREES
 
+#undef RGB
+#define RGB(r,g,b) ((COLORREF)(((DWORD)(r)|((DWORD)((BYTE)(g))<<8))|(((DWORD)(BYTE)(b))<<16)))
 #define FLUSH(w) ;
 #define DC(w,dr) ((dr==0)?w->core.hDC:w->core.memDC)
 #define LINE(w,x1,y1,x2,y2) (void)MoveTO(w,x1,y1,NULL); \
@@ -79,7 +85,7 @@
   (HBITMAP) SelectObject(w->core.memDC,dr); \
   w->core.hPen = CreatePen(PS_SOLID, 1, c); \
   w->core.hOldPen = (HPEN) SelectObject(DC(w,dr), w->core.hPen); \
-  if (x1==x2&&y1==y2) SetPixel(DC(w,dr),x1,y1,c); \
+  SetPixel(DC(w,dr),x2,y2,c); \
   LINE(DC(w,dr),x1,y1,x2,y2); \
   (void) SelectObject(DC(w,dr), w->core.hOldPen); \
   (void) DeleteObject(w->core.hPen); \
@@ -237,6 +243,7 @@ typedef struct _CorePart {
 	HPEN        hOldPen, hPen;
 	HBRUSH      hOldBrush, hBrush;
 	HBITMAP     hOldBitmap;
+	HMENU       hMenu;
 } CorePart;
 
 #else
@@ -247,7 +254,6 @@ typedef struct _CorePart {
 #include <math.h>
 #ifdef VMS
 #include <unixlib.h>
-#define getlogin() cuserid(NULL)
 #else
 #if HAVE_UNISTD_H
 #include <unistd.h>
@@ -318,6 +324,7 @@ typedef struct _CorePart {
 
 extern void intCat(char **string, const char *var1, const int var2);
 extern void stringCat(char **string, const char *var1, const char *var2);
+extern void intCpy(char *string, const int var);
 
 #define ABS(a) (((a)<0)?(-(a)):(a))
 #define SIGN(a) (((a)<0)?(-1):1)
